<?php

include_once("config.php");

class vote{
/*
vote类说明

//Copyleft by Felix021 @ http://www.198880711.com

1.  实例化时会调用构造函数,必须保证包含了config.php, 如: include("config.php");
    config.php中需要包含$dbhost, $dbuser, $dbpass, $dbname, $tblprefix, $interval,
    $votelimit, $starttime, $finishtime, $GMT, $exts, $uploaddir, $map等13个变量.
    
2.  该类使用$map记录字段的名称和数量, 在$map中用//分割字段的名称, 在数据库的
    candidate表中按顺序对应item0, item1, ... , item9. 最多10个字段.
     
2.  使用方法:
     首先安装: 在获得了数据库的用户名和密码后上传投票系统, 执行install.php进行安装该程序会
                自动创建必须的配置文件和数据库中的表
     安装完毕: 为保证安全必须将install.php删除或重命名(至少不应该还是.php文件)

3.  此类的使用方法

    //实例化
    $v1 = new vote;

    //设置初值, 可以只设置前面的值,后面的字段可以空着; 如果不想修改某字段, 则在该
    //位置填入false, 如 $v1->set("1.jpg", false, "2", false, "3");
    $v1->set(图片路径, 字段0的值, 字段1的值, ... 字段9的值);
    
    //从数据库中读取数据进行初始化
    $v1->init(候选人id);
    
    //插入新数据
    $v1->set(xxxxxx);//先设置初值
    $v1->insert();//执行完毕后v1即对应该数据
    
    //更新数据, 
    //需先置候选人id, 如$v1->init(1) 或 $v1->id = 1;
    $v1->set(xxxxxx);//修改值,如果不想修改某字段, 则在该位置填入false
    $v1->update();//更新完毕后v1即对应该数据
    
    //删除数据
    $v1->delete(候选人id);
    e.g. $v1->delete(1);//删除第一个
    
    //取得最后插入的候选人id
    $maxid = $v1->maxid();
    
    //投票. 投票的规则是在$interval(config.php)规定的时间内不能给
    //$votelimit个人投票, 也不能给同一个人投票.    
    $v1->vote(候选人id);
    e.g.  $v1->vote(1); //给第一个人投票
    
    //上传文件
    //先建立表单如下:
    <form enctype="multipart/form-data" method="post" action="Your URL">
        <!-- 注意file要用 name="file"  -->
        <input type="file" name="file"/>
        <input type="submit" value="提交"/>
        <!-- 可继续扩展添加其他数据 -->
    </form>
    //然后在处理表单的位置加上下句即可：
    $v1->uploadfile(候选人id);
    e.g. $v1->uploadfile(1); //上传文件并更新至第一个人的数据中
    
    //列出所有id, 默认案id升序
    $v1->count(排序字段名, 升序/降序);
    e.g.
      $v1->count();//按id升序
      $v1->count("votes", "asc");//按票数升序
      $v1->count("votes, id", "desc");//按先票数后id降序

4.  错误处理:
     应该在每一项操作后检查$v1->valid是否为true; 如果为false, 表示操作失败
     可从$v1->errorstr读取错误信息字符串.

*/
var $conn;  //链接
var $candidatetbl; //候选人数据表名
var $iptbl; //ip数据表名
var $interval; //投票时间间隔
var $valid; //有效性标志(true|false)
var $errorstr; //出错时的错误描述
var $status; //投票状态 0-未开始 1-进行中 2-结束
var $statusstr; //投票状态描述字符串
var $timestr; //间隔时间描述字符串
var $exts; //允许上传的文件扩展名数组
var $countid;
var $ids;

var $id;
var $votes; //票数
var $file; //文件地址
var $itemcount; //设置显示几个字段
var $items; //自定义字段,数组  items[i]['name']为显示名称, items[i]['value']为值, 下同

//构造函数, 设置数据库连接的基本参数
function __construct(){
    global $dbhost, $dbuser, $dbpass, $dbname, $tblprefix, $interval, $starttime, $finishtime, $GMT, $exts, $map;
    if(file_exists("install.php")){
        $this->valid = false;
        $this->errorstr = "为确保安全请及时删除install.php";
        die($this->errorstr);
        return;
    }
    $this->candidatetbl = $tblprefix . "candidate";
    $this->iptbl = $tblprefix . "ip";
    $this->interval = $interval;
    $this->conn = new mysqli($dbhost, $dbuser, $dbpass, $dbname);
    if($this->conn){
        $this->valid = true;
        @$this->conn->query("SET NAMES 'utf8'");
        $current = time();
        $start = strtotime($starttime) + ($GMT+0) * 3600;
        $finish = strtotime($finishtime) + ($GMT+0) * 3600;
        if($current < $start){
            $this->status = 0;
            $this->statusstr = "未开始";
        }else if($current > $finish){
            $this->status = 2;
            $this->statusstr = "已结束";
        }else{
            $this->status = 1;
            $this->statusstr = "进行中";
        }
        
        $this->itemcount = 0;
        $map = explode("//", $map);
        foreach($map as $itemname){
            $itemname = trim($itemname);
            if(empty($itemname)) continue;
            $this->items[$this->itemcount]['name'] = $itemname;
            $this->itemcount++;
            if($this->itemcount == 10)break;//最多10个字段
        }
        
        if($interval < 3600){
            $this->timestr = round($interval / 60, 2) . "分钟";
        }else{
            $h = floor($interval / 3600);
            $m = round(($interval - $h * 3600) / 60, 2);
            $this->timestr = $h . "小时" . $m . "分钟";
        }
        $this->exts = explode(" ", $exts);
        foreach($this->exts as &$ext){
            $ext = trim(strtolower($ext));
        }
        
    }else{
        $this->valid = false;
        $this->errorstr = "Connection failed: " . mysqli_error($this->conn);
    }
}//end of function __construct()

//析构函数,释放数据库资源
function __destruct(){
    @mysqli_close($this->conn);
}//end of function __destruct()

function count($orderby = "id", $order = "asc"){
    $conn = $this->conn;
    $query = "select id from {$this->candidatetbl} where 1 order by {$orderby} {$order}";
    $result = $conn->query($query);
    if($result){
        $this->valid = true;
        $this->countid = $result->num_rows;
        while($row = $result->fetch_assoc()){
            $this->ids[] = $row['id'];
        }
    }else{
        $this->valid = false;
        $this->errorstr = "无法查询数据库";
    }
}

//初始化函数, 根据id获取字段的值
function init($id){
    if($id === ""){
        $this->valid = false;
        $this->errorstr = '调用vote::init($id)时请设置$id';
        return;
    }
    $conn = $this->conn;
    $query = "select * from {$this->candidatetbl} where id={$id}";
    $db = $conn->query($query);
    if($db === false){
        $this->valid = false;
        $this->errorstr = mysqli_error($conn);
        return;
    }else{
        $num_rows = $db->num_rows;
        if($num_rows == 0){
            $this->valid = false;
            $this->errorstr = "无id({$id})对应的数据";
            return;
        }
        $this->valid = true;
        $row = $db->fetch_assoc();
        $this->id = $row['id'];
        $this->votes = $row['votes'];
        $this->file = $row['file'];
        
        for($i = 0; $i < $this->itemcount; $i++){
            $pointer = "item" . $i;
            $this->items[$i]['value'] = $row[$pointer];
        }
    }
}//end of function init()

//返回当前最大的id
function maxid(){
    $conn = $this->conn;
    $query = "select max(id) as maxid from {$this->candidatetbl}";
    $db = $conn->query($query);
    if($db === false){
        $this->valid = false;
        $this->errorstr = mysqli_error($conn);
        return;
    }else{
        $num_rows = $db->num_rows;
        if($num_rows == 0){
            $this->valid = false;
            $this->errorstr = "无对应的数据";
            return;
        }
        $this->valid = true;
        $row = $db->fetch_assoc();
        $maxid = $row['maxid'];
        if(empty($maxid))$maxid = 0;
        return $maxid;
    }
}//end of function maxid()

function set($file = false, $item0 = false, $item1 = false, $item2 = false, $item3 = false, $item4 = false, $item5 = false, $item6 = false, $item7 = false, $item8 = false, $item9 = false){
    if($file !== false) $this->file = $file;
    if($item0 !== false) $this->items[0]['value'] = $item0;
    if($item1 !== false) $this->items[1]['value'] = $item1;
    if($item2 !== false) $this->items[2]['value'] = $item2;
    if($item3 !== false) $this->items[3]['value'] = $item3;
    if($item4 !== false) $this->items[4]['value'] = $item4;
    if($item5 !== false) $this->items[5]['value'] = $item5;
    if($item6 !== false) $this->items[6]['value'] = $item6;
    if($item7 !== false) $this->items[7]['value'] = $item7;
    if($item8 !== false) $this->items[8]['value'] = $item8;
    if($item9 !== false) $this->items[9]['value'] = $item9;
}//end of function set()

//插入函数, 新建一个候选人
function insert(){
    $this->id = $this->maxid() + 1;
    $conn = $this->conn;
    $query = "insert into {$this->candidatetbl} (votes)values(0)";
    $result = $conn->query($query);
    if($result){
        $this->valid = true;
        $this->update();
    }else{
        $this->valid = false;
        $this->errorstr = "无法插入新数据";
    }
}//end of function insert()

//更新函数
function update(){
    $conn = $this->conn;
    if(!empty($this->votes)){
        $query = "update {$this->candidatetbl} set votes={$this->votes} where id={$this->id}";
        $result = $conn->query($query);
        if($result === false){
            $this->valid = false;
            $this->errorstr = "无法更新数据";
            return;
        }
    }
    if(!empty($this->file)){
        $query = "update {$this->candidatetbl} set file='{$this->file}' where id={$this->id}";
        $result = $conn->query($query);
        if($result === false){
            $this->valid = false;
            $this->errorstr = "无法更新数据";
            return;
        }
    }
    for($i = 0; $i < $this->itemcount; $i++){
        if(!empty($this->items[$i]['value'])){
            $query = "update {$this->candidatetbl} set item{$i}='{$this->items[$i]['value']}' where id={$this->id}";
            $result = $conn->query($query);
            if($result === false){
                $this->valid = false;
                $this->errorstr = "无法更新数据";
                return;
            }
        }
    }
    $this->valid = true;
}//end of function update()

function delete($id){
    if($id === ""){
        $this->valid = false;
        $this->errorstr = '调用vote::delete($id)时请设置$id';
        return;
    }
    $conn = $this->conn;
    $query = "delete from {$this->candidatetbl} where id={$id}";
    @$conn->query($query);
    $this->init($id);
    if($this->valid === false){
        $this->valid = true;
    }else{
        $this->errorstr = "无法删除";
    }
}//end of function delete()


function vote($id){
    global $interval, $votelimit;
    $timestr = $this->timestr;
    $this->init($id);
    if($this->valid === false)return;
    if($this->status != 1){
        $this->valid = false;
        $this->errorstr = $this->status == 0 ? "投票尚未开始":"投票已经结束";
        return;
    }
    $conn = $this->conn;
    $ip = $_SERVER['REMOTE_ADDR'];
    $time = time() - $interval;
    $query = "select * from {$this->iptbl} where ip='{$ip}' and time>={$time} order by id desc limit 0, {$votelimit}";
    $result = $conn->query($query);
    if($result === false){
        $this->valid = false;
        $this->errorstr = "读取ip数据出错";
        return;
    }else if($result->num_rows >= $votelimit){
        $this->valid = false;
        $this->errorstr = "{$timestr}内最多只能给{$votelimit}人投票";
        return;
    }else if($result->num_rows > 0){
        while($row = $result->fetch_assoc()){
            $ids[] = $row['cid'];
        }
        if(in_array($id, $ids)){
            $this->valid = false;
            $this->errorstr = "{$timestr}内不能重复给同一候选人投票";
            return;
        }
    }
    
    
    $this->votes++;
    $query = "update {$this->candidatetbl} set votes={$this->votes} where id={$this->id}";
    $result = $conn->query($query);
    if($result === false){
        $this->valid = false;
        $this->errorstr = "无法更新票数";
        return;
    }
    $query = "insert into {$this->iptbl} (cid, ip, time) values ({$id}, '{$ip}', ". time() .")";
    $result = $conn->query($query);
}//end of function vote()


//提取文件扩展名的小函数
function ext($filename){
    $i = strlen($filename);
    while(substr($filename, $i, 1) != "." && $i >= 0)$i--;
    if($i >= 0){
        return substr($filename, $i + 1);
    }else{
        return "";
    }
}//end of function ext()

//处理表单上传文件并更新至数据库
function uploadfile($id, $file = "file"){
    $this->init($id);
    if($this->valid === false) return;
    $ext = strtolower($this->ext($_FILES[$file]['name']));
    if(in_array(strtolower($ext), $this->exts) ===  false){
        $this->valid = false;
        $this->errorstr = "不允许的文件类型: {$ext}";
        return;
    }
    $uploadfile = "attachment/{$id}_" . time() . ".{$ext}";
    if(!file_exists("attachment")) {@mkdir("attachment");@unlink("attachment");}
    if(file_exists($uploadfile)) @unlink($uploadfile);
    $res = move_uploaded_file($_FILES[$file]['tmp_name'], $uploadfile);
    if ($res){
        $query = "update {$this->candidatetbl} set file='{$uploadfile}' where id={$id}";
        $result = $this->conn->query($query);
        if($result){
            $this->valid = true;
            return;
        }else{
            $this->valid = false;
            $this->errorstr = "无法更新数据库";
            return;
        }
    }else{
        $this->valid = false;
        $this->errorstr = "文件上传失败!";
        return;
    }
}//end of function uploadfile()

};//end of class vote

/*
$v1 = new vote;
$v1->count("votes, id", "asc");
echo "<pre>\n\n";var_dump($v1);echo "</pre>";

$v1->set("", "", "2", "3", "4", "5");
$v1->insert();
echo "<pre>\n\n";var_dump($v1);echo "</pre>";
*/

?>

